@@ -53,10 +53,18 @@
 #include <sipwitch/mapped.h>
 #endif
 
-#ifndef	_SIPWITCH_RTPPROXY_H
+#ifndef	_SIPWITCH_RTPPROXY_H_
 #include <sipwitch/rtpproxy.h>
 #endif
 
+#ifndef	_SIPWITCH_PROCESS_H_
+#include <sipwitch/process.h>
+#endif
+
+#ifndef	_SIPWITCH_CDR_H_
+#include <sipwitch/cdr.h>
+#endif
+
 #define	CONFIG_KEY_SIZE 177
 
 NAMESPACE_SIPWITCH
@@ -124,6 +132,7 @@ public:
     {
 	protected:
 		friend class service;
+		friend class modules;
 
 		unsigned runlevel;
 		bool active_flag;
@@ -150,6 +159,9 @@ public:
 		inline bool isActive(void) const
 			{return active_flag;};
 
+		virtual void period(long slice);
+		virtual void cdrlog(cdr *call);
+		virtual void errlog(errlevel_t level, const char *text);
 		virtual bool check(void);
 		virtual void snapshot(FILE *fp);
 		virtual void start(service *cfg);
@@ -194,7 +206,6 @@ public:
 	static FILE *open(const char *uid = NULL, const char *cfgpath = NULL);
 	static void startup(void);
 	static void shutdown(void);
-	static void snmptrap(unsigned id, const char *descr);
 	static long uptime(void);
 	static bool match(const char *digits, const char *pattern, bool partial);
 	static keynode *get(void);
@@ -236,15 +247,6 @@ protected:
 
 private:
 	void __LOCAL addAttributes(keynode *node, char *astr);
-
-	class __LOCAL snmpserver : public LinkedObject
-	{
-	public:
-		struct sockaddr_internet server;
-	};
-
-	snmpserver *snmpservers;
-	const char *community;
 };
 
 #define	RUNLEVELS	(sizeof(callback::runlevels) / sizeof(LinkedObject *))
